@@ -45,6 +45,7 @@
 #include "dialog-commodity.h"
 #include "dialog-totd.h"
 #include "gnc-ui-util.h"
+#include "gnc-uri-utils.h"
 #include "gnc-session.h"
 #include "qofbookslots.h"
 #ifdef G_OS_WIN32
@@ -241,15 +242,15 @@ void
 gnc_gnome_help (const char *dir, const char *detail)
 {
     NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
-NSString *subdir = [NSString stringWithUTF8String: dir];
-NSString *tag, *subdirectory;
+    NSString *subdir = [NSString stringWithUTF8String: dir];
+    NSString *tag, *subdirectory;
     NSURL *url = NULL;
 
     if (detail)
-tag  = [NSString stringWithUTF8String: detail];
-else if ([subdir compare: @HF_HELP] == NSOrderedSame)
+        tag  = [NSString stringWithUTF8String: detail];
+    else if ([subdir compare: @HF_HELP] == NSOrderedSame)
         tag = @"help";
-else if ([subdir compare: @HF_GUIDE] == NSOrderedSame)
+    else if ([subdir compare: @HF_GUIDE] == NSOrderedSame)
         tag = @"index";
     else
     {
@@ -266,18 +267,18 @@ else if ([subdir compare: @HF_GUIDE] == NSOrderedSame)
          * add some more pieces, and put it all back together again. Then,
          * because the gettext way of handling localizations is different from
          * OSX's, we have to figure out which translation to use. */
-NSArray *components = [NSArray arrayWithObjects: @"share", @"doc", @"gnucash-docs", nil ];
+        NSArray *components = [NSArray arrayWithObjects: @"share", @"doc", @"gnucash-docs", nil ];
         NSString *prefix = [[[NSBundle mainBundle] resourcePath]
                             stringByDeletingLastPathComponent];
         NSArray *prefix_comps = [[prefix pathComponents]
-                         arrayByAddingObjectsFromArray: components];
-NSString *docs_dir = [NSString pathWithComponents: prefix_comps];
+                                 arrayByAddingObjectsFromArray: components];
+        NSString *docs_dir = [NSString pathWithComponents: prefix_comps];
         NSArray *languages = [[NSUserDefaults standardUserDefaults]
-                      objectForKey: @"AppleLanguages"];
+                              objectForKey: @"AppleLanguages"];
         BOOL dir;
-subdir = [[[subdir lowercaseString] componentsSeparatedByString: @" "]
-          componentsJoinedByString: @"-"];
-if (![[NSFileManager defaultManager] fileExistsAtPath: docs_dir])
+        subdir = [[[subdir lowercaseString] componentsSeparatedByString: @" "]
+                  componentsJoinedByString: @"-"];
+        if (![[NSFileManager defaultManager] fileExistsAtPath: docs_dir])
         {
             const gchar *message =
                 _("GnuCash could not find the files for the help documentation. "
@@ -296,28 +297,28 @@ if (![[NSFileManager defaultManager] fileExistsAtPath: docs_dir])
                 NSArray *elements;
                 unsigned int paths;
                 NSString *completed_path = [NSString alloc];
-this_lang = [this_lang stringByTrimmingCharactersInSet:
-             [NSCharacterSet characterSetWithCharactersInString:
+                this_lang = [this_lang stringByTrimmingCharactersInSet:
+                             [NSCharacterSet characterSetWithCharactersInString:
                               @"\""]];
-elements = [this_lang componentsSeparatedByString: @"-"];
+                elements = [this_lang componentsSeparatedByString: @"-"];
                 this_lang = [elements objectAtIndex: 0];
-path = [docs_dir stringByAppendingPathComponent: this_lang];
-paths = [path completePathIntoString: &completed_path
-         caseSensitive: FALSE
-         matchesIntoArray: NULL filterTypes: NULL];
+                path = [docs_dir stringByAppendingPathComponent: this_lang];
+                paths = [path completePathIntoString: &completed_path
+                         caseSensitive: FALSE
+                         matchesIntoArray: NULL filterTypes: NULL];
                 if (paths > 1 &&
-                        [[NSFileManager defaultManager]
-         fileExistsAtPath: completed_path
-         isDirectory: &dir])
+                    [[NSFileManager defaultManager]
+                     fileExistsAtPath: completed_path
+                     isDirectory: &dir])
                     if (dir)
                     {
                         @try
                         {
-url = [NSURL fileURLWithPath:
+                            url = [NSURL fileURLWithPath:
                                    [[[completed_path
-          stringByAppendingPathComponent: subdir]
-         stringByAppendingPathComponent: tag]
-        stringByAppendingPathExtension: @"html"]];
+                                      stringByAppendingPathComponent: subdir]
+                                     stringByAppendingPathComponent: tag]
+                                    stringByAppendingPathExtension: @"html"]];
                         }
                         @catch (NSException *e)
                         {
@@ -327,7 +328,7 @@ url = [NSURL fileURLWithPath:
                         }
                         break;
                     }
-if ([this_lang compare: @"en"] == NSOrderedSame)
+                if ([this_lang compare: @"en"] == NSOrderedSame)
                     break; /* Special case, forces use of "C" locale */
             }
         }
@@ -336,11 +337,11 @@ if ([this_lang compare: @"en"] == NSOrderedSame)
             @try
             {
                 url = [NSURL
-       fileURLWithPath: [[[[docs_dir
-                            stringByAppendingPathComponent: @"C"]
-                           stringByAppendingPathComponent: subdir]
-                          stringByAppendingPathComponent: tag]
-                         stringByAppendingPathExtension: @"html"]];
+                       fileURLWithPath: [[[[docs_dir
+                                            stringByAppendingPathComponent: @"C"]
+                                           stringByAppendingPathComponent: subdir]
+                                          stringByAppendingPathComponent: tag]
+                                         stringByAppendingPathExtension: @"html"]];
             }
             @catch (NSException *e)
             {
@@ -355,10 +356,10 @@ if ([this_lang compare: @"en"] == NSOrderedSame)
     {
         @try
         {
-url = [NSURL fileURLWithPath: [[NSBundle mainBundle]
-                               pathForResource: tag
-                               ofType: @"html"
-                               inDirectory: subdir ]];
+            url = [NSURL fileURLWithPath: [[NSBundle mainBundle]
+                                           pathForResource: tag
+                                           ofType: @"html"
+                                           inDirectory: subdir ]];
         }
         @catch (NSException *e)
         {
@@ -369,7 +370,7 @@ url = [NSURL fileURLWithPath: [[NSBundle mainBundle]
     }
     /* Now just open the URL in the default app for opening URLs */
     if (url)
-[[NSWorkspace sharedWorkspace] openURL: url];
+        [[NSWorkspace sharedWorkspace] openURL: url];
     else
     {
         const gchar *message =
@@ -482,32 +483,33 @@ void
 gnc_launch_assoc (const char *uri)
 {
     wchar_t *winuri = NULL;
-    char* scheme = g_uri_parse_scheme(uri);
     /* ShellExecuteW open doesn't decode http escapes if it's passed a
      * file URI so we have to do it. */
-    if (scheme && strcmp (scheme, "file") == 0)
+    if (gnc_uri_is_file_uri (uri))
     {
-	gchar *filename = g_filename_from_uri (uri, NULL, NULL);
-	winuri = (wchar_t *)g_utf8_to_utf16(filename, -1, NULL, NULL, NULL);
+        gchar *uri_ue = g_uri_unescape_string (uri, NULL);
+        gchar *filename = gnc_uri_get_path (uri_ue);
+        winuri = (wchar_t *)g_utf8_to_utf16(filename, -1, NULL, NULL, NULL);
+        g_free (filename);
+        g_free (uri_ue);
     }
     else
-	winuri = (wchar_t *)g_utf8_to_utf16(uri, -1, NULL, NULL, NULL);
+        winuri = (wchar_t *)g_utf8_to_utf16(uri, -1, NULL, NULL, NULL);
 
     if (winuri)
     {
-	wchar_t *wincmd = (wchar_t *)g_utf8_to_utf16("open", -1,
-						     NULL, NULL, NULL);
-	if ((INT_PTR)ShellExecuteW(NULL, wincmd, winuri,
-				   NULL, NULL, SW_SHOWNORMAL) <= 32)
-	{
-	    const gchar *message =
-		_("GnuCash could not find the associated file");
-	    gnc_error_dialog(NULL, "%s: %s", message, uri);
-	}
-	g_free (wincmd);
-	g_free (winuri);
+        wchar_t *wincmd = (wchar_t *)g_utf8_to_utf16("open", -1,
+                                 NULL, NULL, NULL);
+        if ((INT_PTR)ShellExecuteW(NULL, wincmd, winuri,
+                       NULL, NULL, SW_SHOWNORMAL) <= 32)
+        {
+            const gchar *message =
+            _("GnuCash could not find the associated file");
+            gnc_error_dialog(NULL, "%s: %s", message, uri);
+        }
+        g_free (wincmd);
+        g_free (winuri);
     }
-    g_free (scheme);
 }
 
 #else
@@ -647,11 +649,15 @@ gnc_ui_start_event_loop (void)
     id = g_timeout_add_full (G_PRIORITY_DEFAULT_IDLE, 10000, /* 10 secs */
                              gnc_ui_check_events, NULL, NULL);
 
+    scm_call_1(scm_c_eval_string("gnc:set-ui-status"), SCM_BOOL_T);
+
     /* Enter gnome event loop */
     gtk_main ();
 
     g_source_remove (id);
 
+    scm_call_1(scm_c_eval_string("gnc:set-ui-status"), SCM_BOOL_F);
+
     gnome_is_running = FALSE;
     gnome_is_terminating = FALSE;
 
