@@ -67,33 +67,7 @@ typedef struct GncTreeViewCommodityPrivate
 
 static GObjectClass *parent_class = NULL;
 
-GType
-gnc_tree_view_commodity_get_type (void)
-{
-    static GType gnc_tree_view_commodity_type = 0;
-
-    if (gnc_tree_view_commodity_type == 0)
-    {
-        static const GTypeInfo our_info =
-        {
-            sizeof (GncTreeViewCommodityClass),
-            NULL,
-            NULL,
-            (GClassInitFunc) gnc_tree_view_commodity_class_init,
-            NULL,
-            NULL,
-            sizeof (GncTreeViewCommodity),
-            0,
-            (GInstanceInitFunc) gnc_tree_view_commodity_init
-        };
-
-        gnc_tree_view_commodity_type = g_type_register_static (GNC_TYPE_TREE_VIEW,
-                                       "GncTreeViewCommodity",
-                                       &our_info, 0);
-    }
-
-    return gnc_tree_view_commodity_type;
-}
+G_DEFINE_TYPE_WITH_PRIVATE(GncTreeViewCommodity, gnc_tree_view_commodity, GNC_TYPE_TREE_VIEW)
 
 static void
 gnc_tree_view_commodity_class_init (GncTreeViewCommodityClass *klass)
@@ -111,8 +85,6 @@ gnc_tree_view_commodity_class_init (GncTreeViewCommodityClass *klass)
 
     /* GtkWidget signals */
     widget_class->destroy = gnc_tree_view_commodity_destroy;
-
-    g_type_class_add_private(klass, sizeof(GncTreeViewCommodityPrivate));
 }
 
 static void
@@ -381,7 +353,7 @@ gnc_tree_view_commodity_new (QofBook *book,
     /* Set default visibilities */
     gtk_tree_view_set_headers_visible (GTK_TREE_VIEW(view), FALSE);
 
-    col = gnc_tree_view_add_text_column (
+    gnc_tree_view_add_text_column (
               view, _("Namespace"), "namespace", NULL, "NASDAQ",
               GNC_TREE_MODEL_COMMODITY_COL_NAMESPACE,
               GNC_TREE_VIEW_COLUMN_VISIBLE_ALWAYS,
@@ -398,7 +370,7 @@ gnc_tree_view_commodity_new (QofBook *book,
               GNC_TREE_MODEL_COMMODITY_COL_VISIBILITY,
               sort_by_commodity_string);
     g_object_set_data(G_OBJECT(col), DEFAULT_VISIBLE, GINT_TO_POINTER(1));
-    col = gnc_tree_view_add_text_column (
+    gnc_tree_view_add_text_column (
               view, _("Print Name"), "printname", NULL,
               "ACMEACME (Acme Corporation, Inc.)",
               GNC_TREE_MODEL_COMMODITY_COL_PRINTNAME,
@@ -410,7 +382,7 @@ gnc_tree_view_commodity_new (QofBook *book,
               GNC_TREE_MODEL_COMMODITY_COL_VISIBILITY,
               sort_by_commodity_string);
     g_object_set_data(G_OBJECT(col), DEFAULT_VISIBLE, GINT_TO_POINTER(1));
-    col = gnc_tree_view_add_text_column (
+    gnc_tree_view_add_text_column (
               view, _("Unique Name"), "uniquename", NULL,
               "NASDAQ::ACMEACME", GNC_TREE_MODEL_COMMODITY_COL_UNIQUE_NAME,
               GNC_TREE_MODEL_COMMODITY_COL_VISIBILITY,
@@ -418,7 +390,7 @@ gnc_tree_view_commodity_new (QofBook *book,
     col = gnc_tree_view_add_text_column (
               /* Translators: Again replace CUSIP by the name of your
                  National Securities Identifying Number. */
-              view, _("ISIN/CUSIP"), "cusip_code", NULL, "QWERTYUIOP",
+              view, _("ISIN/CUSIP"), "cusip_code", NULL, "US1234567890",
               GNC_TREE_MODEL_COMMODITY_COL_CUSIP,
               GNC_TREE_MODEL_COMMODITY_COL_VISIBILITY,
               sort_by_commodity_string);
@@ -430,7 +402,7 @@ gnc_tree_view_commodity_new (QofBook *book,
               GNC_TREE_MODEL_COMMODITY_COL_VISIBILITY,
               sort_by_fraction);
     g_object_set_data(G_OBJECT(col), DEFAULT_VISIBLE, GINT_TO_POINTER(1));
-    col = gnc_tree_view_add_toggle_column(
+    gnc_tree_view_add_toggle_column(
               view, _("Get Quotes"),
               /* Translators: This string has a context prefix; the translation
                  must only contain the part after the | character. */
@@ -439,8 +411,8 @@ gnc_tree_view_commodity_new (QofBook *book,
               GNC_TREE_MODEL_COMMODITY_COL_VISIBILITY,
               sort_by_quote_flag,
               NULL);
-    col = gnc_tree_view_add_text_column (
-              view, _("Source"), "quote_source", NULL, "yahoo",
+    gnc_tree_view_add_text_column (
+              view, _("Source"), "quote_source", NULL, "alphavantage",
               GNC_TREE_MODEL_COMMODITY_COL_QUOTE_SOURCE,
               GNC_TREE_MODEL_COMMODITY_COL_VISIBILITY,
               sort_by_commodity_string);
