@@ -163,7 +163,7 @@ static GtkActionEntry gnc_plugin_page_budget_actions [] =
 static guint gnc_plugin_page_budget_n_actions =
     G_N_ELEMENTS (gnc_plugin_page_budget_actions);
 
-#if 0 
+#if 0
 static const gchar *actions_requiring_account[] =
 {
     "OpenAccountAction",
@@ -175,10 +175,10 @@ static const gchar *actions_requiring_account[] =
 /** Short labels for use on the toolbar buttons. */
 static action_toolbar_labels toolbar_labels[] =
 {
-    { "OpenAccountAction", 	    N_("Open") },
-    { "DeleteBudgetAction", 	    N_("Delete") },
-    { "OptionsBudgetAction", 	    N_("Options") },
-    { "EstimateBudgetAction", 	    N_("Estimate") },
+    { "OpenAccountAction",          N_("Open") },
+    { "DeleteBudgetAction",         N_("Delete") },
+    { "OptionsBudgetAction",        N_("Options") },
+    { "EstimateBudgetAction",       N_("Estimate") },
     { NULL, NULL },
 };
 
@@ -206,41 +206,13 @@ typedef struct GncPluginPageBudgetPrivate
     gint sigFigs;
 } GncPluginPageBudgetPrivate;
 
+G_DEFINE_TYPE_WITH_PRIVATE(GncPluginPageBudget, gnc_plugin_page_budget, GNC_TYPE_PLUGIN_PAGE)
+
 #define GNC_PLUGIN_PAGE_BUDGET_GET_PRIVATE(o)  \
    (G_TYPE_INSTANCE_GET_PRIVATE ((o), GNC_TYPE_PLUGIN_PAGE_BUDGET, GncPluginPageBudgetPrivate))
 
 static GObjectClass *parent_class = NULL;
 
-
-GType
-gnc_plugin_page_budget_get_type (void)
-{
-    static GType gnc_plugin_page_budget_type = 0;
-
-    if (gnc_plugin_page_budget_type == 0)
-    {
-        static const GTypeInfo our_info =
-        {
-            sizeof (GncPluginPageBudgetClass),
-            NULL,
-            NULL,
-            (GClassInitFunc) gnc_plugin_page_budget_class_init,
-            NULL,
-            NULL,
-            sizeof (GncPluginPageBudget),
-            0,
-            (GInstanceInitFunc) gnc_plugin_page_budget_init
-        };
-
-        gnc_plugin_page_budget_type =
-            g_type_register_static (GNC_TYPE_PLUGIN_PAGE,
-                                    "GncPluginPageBudget", &our_info, 0);
-    }
-
-    return gnc_plugin_page_budget_type;
-}
-
-
 GncPluginPage *
 gnc_plugin_page_budget_new (GncBudget *budget)
 {
@@ -295,8 +267,6 @@ gnc_plugin_page_budget_class_init (GncPluginPageBudgetClass *klass)
     gnc_plugin_class->destroy_widget  = gnc_plugin_page_budget_destroy_widget;
     gnc_plugin_class->save_page       = gnc_plugin_page_budget_save_page;
     gnc_plugin_class->recreate_page   = gnc_plugin_page_budget_recreate_page;
-
-    g_type_class_add_private(klass, sizeof(GncPluginPageBudgetPrivate));
 }
 
 
@@ -334,6 +304,7 @@ gnc_plugin_page_budget_init (GncPluginPageBudget *plugin_page)
     /* Visible types */
     priv->fd.visible_types = -1; /* Start with all types */
     priv->fd.show_hidden = FALSE;
+    priv->fd.show_unused = TRUE;
     priv->fd.show_zero_total = TRUE;
     priv->fd.filter_override = g_hash_table_new (g_direct_hash, g_direct_equal);
 
@@ -368,6 +339,22 @@ gnc_plugin_page_budget_close_cb (gpointer user_data)
 }
 
 
+gboolean
+gnc_plugin_page_budget_focus (GncPluginPageBudget *page)
+{
+    if (GNC_IS_PLUGIN_PAGE_BUDGET(page))
+    {
+        GncPluginPageBudgetPrivate *priv = GNC_PLUGIN_PAGE_BUDGET_GET_PRIVATE(page);
+        GncBudgetView *budget_view = priv->budget_view;
+        GtkWidget *account_view = gnc_budget_view_get_account_tree_view (budget_view);
+
+        if (!gtk_widget_is_focus (GTK_WIDGET(account_view)))
+            gtk_widget_grab_focus (GTK_WIDGET(account_view));
+    }
+    return FALSE;
+}
+
+
 static void
 gnc_plugin_page_budget_refresh_cb(GHashTable *changes, gpointer user_data)
 {
@@ -458,8 +445,14 @@ gnc_plugin_page_budget_destroy_widget (GncPluginPage *plugin_page)
     ENTER("page %p", plugin_page);
     priv = GNC_PLUGIN_PAGE_BUDGET_GET_PRIVATE(plugin_page);
 
+    // Remove the page focus idle function if present
+    g_idle_remove_by_data (GNC_PLUGIN_PAGE_BUDGET (plugin_page));
+
     if (priv->budget_view)
     {
+        // save the account filter state information to budget section
+        gnc_budget_view_save_account_filter (priv->budget_view);
+
         if (priv->delete_budget)
         {
             gnc_budget_view_delete_budget (priv->budget_view);
@@ -518,7 +511,7 @@ gnc_plugin_page_budget_save_page (GncPluginPage *plugin_page,
     guid_to_string_buff(gnc_budget_get_guid(priv->budget), guid_str);
     g_key_file_set_string(key_file, group_name, BUDGET_GUID, guid_str);
 
-    //FIXME
+    // Save the Budget page information to state file
     gnc_budget_view_save(priv->budget_view, key_file, group_name);
 
     LEAVE(" ");
@@ -769,8 +762,7 @@ gnc_plugin_page_budget_cmd_view_options (GtkAction *action,
 
         priv->dialog = GTK_WIDGET(gtk_builder_get_object (builder, "budget_options_container_dialog"));
 
-        gtk_window_set_transient_for(
-            GTK_WINDOW(priv->dialog),
+        gtk_window_set_transient_for(GTK_WINDOW(priv->dialog),
             GTK_WINDOW(gnc_plugin_page_get_window(GNC_PLUGIN_PAGE(page))));
 
         gbname = GTK_WIDGET(gtk_builder_get_object (builder, "BudgetName"));
@@ -923,8 +915,7 @@ gnc_plugin_page_budget_cmd_estimate_budget(GtkAction *action,
 
     dialog = GTK_WIDGET(gtk_builder_get_object (builder, "budget_estimate_dialog"));
 
-    gtk_window_set_transient_for(
-        GTK_WINDOW(dialog),
+    gtk_window_set_transient_for(GTK_WINDOW(dialog),
         GTK_WINDOW(gnc_plugin_page_get_window(GNC_PLUGIN_PAGE(page))));
 
     hb = GTK_WIDGET(gtk_builder_get_object (builder, "StartDate_hbox"));
