@@ -50,6 +50,8 @@
 #include "gnc-pricedb.h"
 #include "gnc-ui-util.h"
 #include "gnc-ui.h"
+#include "gnc-uri-utils.h"
+#include "gnc-filepath-utils.h"
 #include "gnc-warnings.h"
 #include "gnucash-sheet.h"
 #include "gnucash-register.h"
@@ -60,13 +62,13 @@
 // static QofLogModule log_module = GNC_MOD_SX;
 static QofLogModule log_module = GNC_MOD_GUI;
 
-#define STATE_SECTION_REG_PREFIX "Register"
-
 /***** PROTOTYPES ***************************************************/
 void gnc_split_reg_raise( GNCSplitReg *gsr );
 
-static GtkWidget* add_summary_label( GtkWidget *summarybar,
-                                     const char *label_str );
+static GtkWidget* add_summary_label( GtkWidget *summarybar, gboolean pack_start,
+                                     const char *label_str, GtkWidget *extra );
+
+static void gsr_summarybar_set_arrow_draw (GNCSplitReg *gsr);
 
 static void gnc_split_reg_determine_read_only( GNCSplitReg *gsr );
 
@@ -159,7 +161,6 @@ void gnc_split_reg_sort_action_cb (GtkWidget *w, gpointer data);
 void gnc_split_reg_sort_notes_cb (GtkWidget *w, gpointer data);
 
 
-void gnc_split_reg_destroy_cb(GtkWidget *widget, gpointer data);
 void gnc_split_reg_size_allocate( GtkWidget *widget,
                                   GtkAllocation *allocation,
                                   gpointer user_data );
@@ -168,6 +169,7 @@ void gnc_split_reg_size_allocate( GtkWidget *widget,
 static void gnc_split_reg_class_init( GNCSplitRegClass *klass );
 static void gnc_split_reg_init( GNCSplitReg *gsr );
 static void gnc_split_reg_init2( GNCSplitReg *gsr );
+void gnc_split_reg_dispose(GObject *obj);
 
 FROM_STRING_FUNC(SortType, ENUM_LIST_SORTTYPE)
 AS_STRING_FUNC(SortType, ENUM_LIST_SORTTYPE)
@@ -314,6 +316,8 @@ gnc_split_reg_class_init( GNCSplitRegClass *klass )
     klass->help_changed_cb = NULL;
     klass->show_popup_menu_cb = NULL;
     klass->include_date_cb = NULL;
+
+    object_class->dispose = gnc_split_reg_dispose;
 }
 
 GtkWidget*
@@ -347,13 +351,20 @@ gnc_split_reg_init( GNCSplitReg *gsr )
     gtk_orientable_set_orientation (GTK_ORIENTABLE(gsr), GTK_ORIENTATION_VERTICAL);
 
     gsr->sort_type = BY_STANDARD;
+    gsr->sort_rev = FALSE;
+    gsr->sort_arrow_handler_id = 0;
+    gsr->filter_text = NULL;
     gsr->width = -1;
     gsr->height = -1;
     gsr->numRows = 10;
     gsr->read_only = FALSE;
+}
 
-    g_signal_connect( gsr, "destroy",
-                      G_CALLBACK (gnc_split_reg_destroy_cb), gsr );
+static void
+gnc_split_reg_pref_acc_labels (gpointer prefs, gchar *pref, gpointer user_data)
+{
+    GNCSplitReg *gsr = user_data;
+    gnucash_register_refresh_from_prefs (gsr->reg);
 }
 
 static void
@@ -367,6 +378,11 @@ gnc_split_reg_init2( GNCSplitReg *gsr )
     /* ordering is important here... setup_status before create_table */
     gsr_create_table( gsr );
     gsr_setup_table( gsr );
+
+    gnc_prefs_register_cb (GNC_PREFS_GROUP_GENERAL,
+                           GNC_PREF_ACCOUNTING_LABELS,
+                           gnc_split_reg_pref_acc_labels,
+                           gsr);
 }
 
 static
@@ -441,8 +457,25 @@ gsr_setup_status_widgets( GNCSplitReg *gsr )
 }
 
 void
-gnc_split_reg_destroy_cb(GtkWidget *widget, gpointer data)
+gnc_split_reg_dispose(GObject *obj)
 {
+    GNCSplitReg *gsr = GNC_SPLIT_REG(obj);
+
+    if (gsr->filter_text)
+        g_free (gsr->filter_text);
+    gsr->filter_text = NULL;
+
+    gnc_prefs_remove_cb_by_func (GNC_PREFS_GROUP_GENERAL,
+                                 GNC_PREF_ACCOUNTING_LABELS,
+                                 gnc_split_reg_pref_acc_labels,
+                                 gsr);
+
+    if (gsr->reg)
+    {
+        g_signal_handlers_disconnect_by_data (gsr->reg, gsr);
+        gtk_widget_destroy (GTK_WIDGET (gsr->reg));
+    }
+    gsr->reg = NULL;
 }
 
 /**
@@ -531,21 +564,109 @@ gsr_redraw_all_cb (GnucashRegister *g_reg, gpointer data)
     print_info = gnc_account_print_info( leader, TRUE );
     reverse = gnc_reverse_balance( leader );
 
-    gsr_update_summary_label( gsr->balance_label,
-                              xaccAccountGetPresentBalance,
-                              leader, print_info, commodity, reverse, euro );
-    gsr_update_summary_label( gsr->cleared_label,
-                              xaccAccountGetClearedBalance,
-                              leader, print_info, commodity, reverse, euro );
-    gsr_update_summary_label( gsr->reconciled_label,
-                              xaccAccountGetReconciledBalance,
-                              leader, print_info, commodity, reverse, euro );
-    gsr_update_summary_label( gsr->future_label,
-                              xaccAccountGetBalance,
-                              leader, print_info, commodity, reverse, euro );
-    gsr_update_summary_label( gsr->projectedminimum_label,
-                              xaccAccountGetProjectedMinimumBalance,
-                              leader, print_info, commodity, reverse, euro );
+    if (gsr->balance_label != NULL) // only test the first as they are a group
+    {
+        gsr_update_summary_label( gsr->balance_label,
+                                  xaccAccountGetPresentBalance,
+                                  leader, print_info, commodity, reverse, euro );
+        gsr_update_summary_label( gsr->cleared_label,
+                                  xaccAccountGetClearedBalance,
+                                  leader, print_info, commodity, reverse, euro );
+        gsr_update_summary_label( gsr->reconciled_label,
+                                  xaccAccountGetReconciledBalance,
+                                  leader, print_info, commodity, reverse, euro );
+        gsr_update_summary_label( gsr->future_label,
+                                  xaccAccountGetBalance,
+                                  leader, print_info, commodity, reverse, euro );
+        gsr_update_summary_label( gsr->projectedminimum_label,
+                                  xaccAccountGetProjectedMinimumBalance,
+                                  leader, print_info, commodity, reverse, euro );
+    }
+
+    // Sort label
+    if (gsr->sort_label != NULL)
+    {
+        gchar *old_tt_text = gtk_widget_get_tooltip_text (GTK_WIDGET(gsr->sort_label));
+        gchar *new_tt_text;
+        gchar *text = NULL;
+
+        switch (gsr->sort_type)
+        {
+        case (0):
+            text =  _("None");
+            break;
+        case (1):
+            text = _("Standard Order");
+            break;
+        case (2):
+            text = _("Date");
+            break;
+        case (3):
+            text = _("Date of Entry");
+            break;
+        case (4):
+            text = _("Statement Date");
+            break;
+        case (5):
+            text = _("Number");
+            break;
+        case (6):
+            text = _("Amount");
+            break;
+        case (7):
+            text = _("Memo");
+            break;
+        case (8):
+            text = _("Description");
+            break;
+        case (9):
+            text = _("Action");
+            break;
+        case (10):
+            text = _("Notes");
+            break;
+        }
+
+        if (gsr->sort_rev)
+            gtk_widget_set_tooltip_text (GTK_WIDGET(gsr->sort_label), _("Descending"));
+        else
+            gtk_widget_set_tooltip_text (GTK_WIDGET(gsr->sort_label), _("Ascending"));
+
+        new_tt_text = gtk_widget_get_tooltip_text (GTK_WIDGET(gsr->sort_label));
+
+        // does the arrow need changing
+        if (g_strcmp0 (old_tt_text, new_tt_text) != 0)
+            gsr_summarybar_set_arrow_draw (gsr);
+
+        if (old_tt_text)
+            g_free (old_tt_text);
+
+        if (new_tt_text)
+            g_free (new_tt_text);
+
+        gtk_label_set_text (GTK_LABEL(gsr->sort_label), text);
+    }
+
+    // Filter label
+    if (gsr->filter_label != NULL)
+    {
+        gchar *old_tt_text = gtk_widget_get_tooltip_text (GTK_WIDGET(gsr->filter_label));
+
+        // check for a change in text
+        if (g_strcmp0 (old_tt_text, gsr->filter_text) != 0)
+        {
+            if (gsr->filter_text != NULL)
+                gtk_label_set_text (GTK_LABEL(gsr->filter_label), _("Filtered"));
+            else
+                gtk_label_set_text (GTK_LABEL(gsr->filter_label), "");
+
+            gtk_widget_set_tooltip_text (GTK_WIDGET(gsr->filter_label), gsr->filter_text);
+
+            if (old_tt_text)
+                g_free (old_tt_text);
+        }
+    }
+
     if (gsr->shares_label == NULL && gsr->value_label == NULL)
         return;
     amount = xaccAccountGetBalance( leader );
@@ -569,8 +690,11 @@ gsr_redraw_all_cb (GnucashRegister *g_reg, gpointer data)
         QofBook *book = gnc_account_get_book (leader);
         GNCPriceDB *pricedb = gnc_pricedb_get_db (book);
         gnc_commodity *currency = gnc_default_currency ();
+        gnc_numeric value =
+            gnc_pricedb_convert_balance_latest_price (pricedb, amount,
+                                                      commodity, currency);
         print_info = gnc_commodity_print_info (currency, TRUE);
-        xaccSPrintAmount (string, amount, print_info);
+        xaccSPrintAmount (string, value, print_info);
         gnc_set_label_color (gsr->value_label, amount);
         gtk_label_set_text (GTK_LABEL (gsr->value_label), string);
 
@@ -586,11 +710,16 @@ gnc_split_reg_ld_destroy( GNCLedgerDisplay *ledger )
     const GncGUID * guid = xaccAccountGetGUID(account);
     gchar guidstr[GUID_ENCODING_LENGTH+1];
     gchar *state_section;
-
+    gchar *acct_fullname;
     guid_to_string_buff(guid, guidstr);
 
     state_section = g_strconcat (STATE_SECTION_REG_PREFIX, " ", guidstr, NULL);
 
+    if (g_strcmp0(guidstr, "00000000000000000000000000000000") == 0)
+        acct_fullname = g_strdup(_("General Journal"));
+    else
+        acct_fullname = gnc_account_get_full_name(account);
+
     if (gsr)
     {
         SplitRegister *reg;
@@ -598,7 +727,7 @@ gnc_split_reg_ld_destroy( GNCLedgerDisplay *ledger )
         reg = gnc_ledger_display_get_split_register (ledger);
 
         if (reg && reg->table)
-            gnc_table_save_state (reg->table, state_section);
+            gnc_table_save_state (reg->table, state_section, acct_fullname);
 
         /*
          * Don't destroy the window here any more.  The register no longer
@@ -606,7 +735,10 @@ gnc_split_reg_ld_destroy( GNCLedgerDisplay *ledger )
          */
     }
     g_free (state_section);
+    g_free (acct_fullname);
+
     gnc_ledger_display_set_user_data (ledger, NULL);
+    g_object_unref (gsr);
 }
 
 void
@@ -792,7 +924,7 @@ gnc_split_reg_reverse_trans_cb (GtkWidget *w, gpointer data)
 
 
 static gboolean
-is_trans_readonly_and_warn (GtkWindow *parent, const Transaction *trans)
+is_trans_readonly_and_warn (GtkWindow *parent, Transaction *trans)
 {
     GtkWidget *dialog;
     const gchar *reason;
@@ -910,7 +1042,7 @@ gsr_default_associate_handler_file (GNCSplitReg *gsr, Transaction *trans, gboole
 {
     GtkWidget *dialog;
     gint       response;
-    gboolean   valid_path_head = FALSE;
+    gboolean   path_head_set = FALSE;
     gchar     *path_head = gnc_prefs_get_string (GNC_PREFS_GROUP_GENERAL, "assoc-head");
 
     dialog = gtk_file_chooser_dialog_new (_("Associate File with Transaction"),
@@ -923,33 +1055,49 @@ gsr_default_associate_handler_file (GNCSplitReg *gsr, Transaction *trans, gboole
 
     gtk_file_chooser_set_local_only (GTK_FILE_CHOOSER(dialog), FALSE);
 
-    if ((path_head != NULL) && (g_strcmp0 (path_head, "") != 0)) // not default entry
-        valid_path_head = TRUE;
+    path_head_set = (path_head && *path_head != '\0'); // not default entry
 
     if (have_uri)
     {
-        gchar *new_uri;
-        gchar *uri_label;
-        gchar *filename;
-
+        gchar *file_uri = NULL;
         const gchar *uri = xaccTransGetAssociation (trans);
+        gchar *scheme = gnc_uri_get_scheme (uri);
 
-        if (valid_path_head && g_str_has_prefix (uri,"file:/") && !g_str_has_prefix (uri,"file://"))
+        if (!scheme) // relative path
         {
-            const gchar *part = uri + strlen ("file:");
-            new_uri = g_strconcat (path_head, part, NULL);
+            gchar *file_path = NULL;
+            if (path_head_set) // not default entry
+                file_path = gnc_file_path_absolute (gnc_uri_get_path (path_head), uri);
+            else
+                file_path = gnc_file_path_absolute (NULL, uri);
+
+            file_uri = gnc_uri_create_uri ("file", NULL, 0, NULL, NULL, file_path);
+            g_free (file_path);
         }
-        else
-            new_uri = g_strdup (uri);
 
-        filename = g_uri_unescape_string (new_uri, NULL);
-        uri_label = g_strconcat (_("Existing Association is "), filename, NULL);
-        gtk_file_chooser_set_extra_widget (GTK_FILE_CHOOSER(dialog), gtk_label_new (uri_label));
-        gtk_file_chooser_set_uri (GTK_FILE_CHOOSER(dialog), new_uri);
+        if (g_strcmp0 (scheme, "file") == 0) // absolute path
+            file_uri = g_strdup (uri);
 
-        g_free (uri_label);
-        g_free (new_uri);
-        g_free (filename);
+        if (file_uri)
+        {
+            GtkWidget *label;
+            gchar *file_uri_u = g_uri_unescape_string (file_uri, NULL);
+            gchar *filename = gnc_uri_get_path (file_uri_u);
+            gchar *uri_label = g_strconcat (_("Existing Association is '"), filename, "'", NULL);
+            PINFO("Path head: '%s', URI: '%s', Filename: '%s'", path_head, uri, filename);
+            label = gtk_label_new (uri_label);
+            gtk_file_chooser_set_extra_widget (GTK_FILE_CHOOSER(dialog), label);
+            gtk_label_set_ellipsize (GTK_LABEL(label), PANGO_ELLIPSIZE_START);
+
+            // Set the style context for this label so it can be easily manipulated with css
+            gnc_widget_set_style_context (GTK_WIDGET(label), "gnc-class-highlight");
+            gtk_file_chooser_set_uri (GTK_FILE_CHOOSER(dialog), file_uri);
+
+            g_free (uri_label);
+            g_free (filename);
+            g_free (file_uri_u);
+            g_free (file_uri);
+        }
     }
     response = gtk_dialog_run (GTK_DIALOG (dialog));
 
@@ -960,28 +1108,63 @@ gsr_default_associate_handler_file (GNCSplitReg *gsr, Transaction *trans, gboole
     {
         gchar *dialog_uri = gtk_file_chooser_get_uri (GTK_FILE_CHOOSER (dialog));
 
-        PINFO("Dialog File URI: %s\n", dialog_uri);
-
-        if (valid_path_head && g_str_has_prefix (dialog_uri, path_head))
+        // prior to 3.5, assoc-head could be with or without a trailing '/'
+        if (path_head_set && !g_str_has_suffix (path_head, "/"))
         {
-            gchar *part = dialog_uri + strlen (path_head);
-            gchar *new_uri = g_strconcat ("file:", part, NULL);
-            xaccTransSetAssociation (trans, new_uri);
-            g_free (new_uri);
+            gchar *folder_with_slash = g_strconcat (path_head, "/", NULL);
+            g_free (path_head);
+            path_head = g_strdup (folder_with_slash);
+            g_free (folder_with_slash);
+
+            if (!gnc_prefs_set_string (GNC_PREFS_GROUP_GENERAL, "assoc-head", path_head))
+                PINFO("Failed to save preference at %s, %s with %s",
+                       GNC_PREFS_GROUP_GENERAL, "assoc-head", path_head);
+        }
+
+        PINFO("Dialog File URI: '%s', Path head: '%s'", dialog_uri, path_head);
+
+        // relative paths do not start with a '/'
+        if (path_head_set && g_str_has_prefix (dialog_uri, path_head))
+        {
+            const gchar *part = dialog_uri + strlen (path_head);
+
+            PINFO("Dialog URI: '%s', Part: '%s'", dialog_uri, part);
+            xaccTransSetAssociation (trans, part);
         }
         else
+        {
+            PINFO("Dialog URI: '%s'", dialog_uri);
             xaccTransSetAssociation (trans, dialog_uri);
-
+        }
         g_free (dialog_uri);
     }
-    g_free (path_head);
+
     gtk_widget_destroy (dialog);
 }
 
+static void
+gsr_default_associate_handler_location_ok_cb (GtkEditable *editable, gpointer user_data)
+{
+    GtkWidget *ok_button = user_data;
+    gboolean have_scheme = FALSE;
+    gchar *text = gtk_editable_get_chars (editable, 0, -1);
+    gchar *scheme;
+
+    if (text && *text != '\0')
+    {
+        scheme = gnc_uri_get_scheme (text);
+        if (scheme)
+            have_scheme = TRUE;
+        g_free (scheme);
+    }
+    gtk_widget_set_sensitive (ok_button, have_scheme);
+    g_free (text);
+}
+
 static void
 gsr_default_associate_handler_location (GNCSplitReg *gsr, Transaction *trans, gboolean have_uri)
 {
-    GtkWidget *dialog, *entry, *label, *content_area;
+    GtkWidget *dialog, *entry, *label, *content_area, *ok_button;
     gint response;
 
     dialog = gtk_dialog_new_with_buttons (_("Associate Location with Transaction"),
@@ -989,9 +1172,12 @@ gsr_default_associate_handler_location (GNCSplitReg *gsr, Transaction *trans, gb
                                      GTK_DIALOG_MODAL,
                                      _("_Remove"), GTK_RESPONSE_REJECT,
                                      _("_Cancel"), GTK_RESPONSE_CANCEL,
-                                     _("_OK"), GTK_RESPONSE_ACCEPT,
+                                     // OK Button added below
                                      NULL);
 
+    ok_button = gtk_dialog_add_button (GTK_DIALOG(dialog), _("_OK"), GTK_RESPONSE_ACCEPT);
+    gtk_widget_set_sensitive (ok_button, FALSE);
+
     content_area = gtk_dialog_get_content_area (GTK_DIALOG (dialog));
 
     // add the entry text
@@ -999,6 +1185,9 @@ gsr_default_associate_handler_location (GNCSplitReg *gsr, Transaction *trans, gb
     gtk_entry_set_width_chars (GTK_ENTRY (entry), 80);
     gtk_entry_set_activates_default (GTK_ENTRY (entry), TRUE);
 
+    g_signal_connect (entry, "changed",
+        G_CALLBACK(gsr_default_associate_handler_location_ok_cb), ok_button);
+
     // add a label and set entry text if required
     if (have_uri)
     {
@@ -1006,7 +1195,7 @@ gsr_default_associate_handler_location (GNCSplitReg *gsr, Transaction *trans, gb
         gtk_entry_set_text (GTK_ENTRY (entry), xaccTransGetAssociation (trans));
     }
     else
-        label = gtk_label_new (_("Enter URL:"));
+        label = gtk_label_new (_("Enter URL like http://www.gnucash.org:"));
 
     // pack label and entry to content area
     gnc_label_set_alignment (label, 0.0, 0.5);
@@ -1030,12 +1219,40 @@ gsr_default_associate_handler_location (GNCSplitReg *gsr, Transaction *trans, gb
     if (response == GTK_RESPONSE_ACCEPT)
     {
         const gchar *dialog_uri = gtk_entry_get_text (GTK_ENTRY (entry));
+
         DEBUG("Location URI: %s\n", dialog_uri);
         xaccTransSetAssociation (trans, dialog_uri);
     }
     gtk_widget_destroy (dialog);
 }
 
+static gchar*
+gsr_convert_associate_uri (Transaction *trans)
+{
+    const gchar *uri = xaccTransGetAssociation (trans); // get the existing uri
+    const gchar *part = NULL;
+
+    if (!uri)
+        return NULL;
+
+    if (g_str_has_prefix (uri, "file:") && !g_str_has_prefix (uri,"file://"))
+    {
+        // fix an error when storing relative paths in version earlier than 3.5
+        // relative paths are stored without a leading "/" and in native form
+        if (g_str_has_prefix (uri,"file:/") && !g_str_has_prefix (uri,"file://"))
+            part = uri + strlen ("file:/");
+        else if (g_str_has_prefix (uri,"file:") && !g_str_has_prefix (uri,"file://"))
+            part = uri + strlen ("file:");
+
+        if (part)
+        {
+            xaccTransSetAssociation (trans, part);
+            return g_strdup (part);
+        }
+    }
+    return g_strdup (uri);
+}
+
 /**
  * Associates a URI with the current transaction.
  **/
@@ -1050,7 +1267,7 @@ gsr_default_associate_handler (GNCSplitReg *gsr, gboolean uri_is_file)
     gboolean have_uri = FALSE;
 
     /* get the current split based on cursor position */
-    if (split == NULL)
+    if (!split)
     {
         gnc_split_register_cancel_cursor_split_changes (reg);
         return;
@@ -1062,21 +1279,24 @@ gsr_default_associate_handler (GNCSplitReg *gsr, gboolean uri_is_file)
     if (cursor_class == CURSOR_CLASS_NONE)
         return;
 
+    // fix an earlier error when storing relative paths in version 3.3
+    uri = gsr_convert_associate_uri (trans);
+
     if (is_trans_readonly_and_warn (GTK_WINDOW(gsr->window), trans))
         return;
 
-    // get the existing uri
-    uri = xaccTransGetAssociation (trans);
-
     // Check for uri is empty or NULL
-    if (g_strcmp0 (uri, "") != 0 && g_strcmp0 (uri, NULL) != 0)
+    if (uri && *uri != '\0')
     {
+        gchar *scheme = gnc_uri_get_scheme (uri);
         have_uri = TRUE;
 
-        if (g_str_has_prefix (uri, "file:")) // use the correct dialog
+        if (!scheme || g_strcmp0 (scheme, "file") == 0) // use the correct dialog
             uri_is_file = TRUE;
         else
             uri_is_file = FALSE;
+
+        g_free (scheme);
     }
 
     if (uri_is_file == TRUE)
@@ -1096,11 +1316,11 @@ gsr_default_execassociated_handler (GNCSplitReg *gsr, gpointer data)
     Transaction *trans;
     Split *split = gnc_split_register_get_current_split (reg);
     const char *uri;
-    const char *run_uri;
+    const char *run_uri = NULL;
     gchar *uri_scheme;
 
     /* get the current split based on cursor position */
-    if (split == NULL)
+    if (!split)
     {
         gnc_split_register_cancel_cursor_split_changes (reg);
         return;
@@ -1117,30 +1337,36 @@ gsr_default_execassociated_handler (GNCSplitReg *gsr, gpointer data)
         xaccTransDump (trans, "ExecAssociated");
 #endif
 
-    uri = xaccTransGetAssociation (trans);
+    // fix an earlier error when storing relative paths in version 3.3
+    uri = gsr_convert_associate_uri (trans);
 
-    if (g_strcmp0 (uri, "") == 0 && g_strcmp0 (uri, NULL) == 0)
+    if (!uri && g_strcmp0 (uri, "") == 0)
         gnc_error_dialog (GTK_WINDOW (gsr->window), "%s", _("This transaction is not associated with a URI."));
     else
     {
-        if (g_str_has_prefix (uri,"file:/") && !g_str_has_prefix (uri,"file://")) // Check for relative path
+        gchar *scheme = gnc_uri_get_scheme (uri);
+
+        if (!scheme) // relative path
         {
             gchar *path_head = gnc_prefs_get_string (GNC_PREFS_GROUP_GENERAL, "assoc-head");
+            gchar *file_path;
 
-            if ((path_head != NULL) && (g_strcmp0 (path_head, "") != 0)) // not default entry
-            {
-                const gchar *part = uri + strlen ("file:");
-                run_uri = g_strconcat (path_head, part, NULL);
-            }
+            if (path_head && g_strcmp0 (path_head, "") != 0) // not default entry
+                file_path = gnc_file_path_absolute (gnc_uri_get_path (path_head), uri);
             else
-                run_uri = g_strdup (uri);
+                file_path = gnc_file_path_absolute (NULL, uri);
+
+            run_uri = gnc_uri_create_uri ("file", NULL, 0, NULL, NULL, file_path);
+            g_free (path_head);
+            g_free (file_path);
         }
-        else
+
+        if (!run_uri)
             run_uri = g_strdup (uri);
 
-        uri_scheme = g_uri_parse_scheme (run_uri);
+        uri_scheme = gnc_uri_get_scheme (run_uri);
 
-        if (uri_scheme != NULL) // make sure we have a scheme entry
+        if (uri_scheme) // make sure we have a scheme entry
         {
             gnc_launch_assoc (run_uri);
             g_free (uri_scheme);
@@ -1175,6 +1401,10 @@ gsr_default_delete_handler( GNCSplitReg *gsr, gpointer data )
     trans = xaccSplitGetParent(split);
     cursor_class = gnc_split_register_get_current_cursor_class (reg);
 
+    /* test for blank_split reference pointing to split */
+    if (gnc_split_register_is_blank_split (reg, split))
+        gnc_split_register_change_blank_split_ref (reg, split);
+
     /* Deleting the blank split just cancels */
     {
         Split *blank_split = gnc_split_register_get_blank_split (reg);
@@ -1366,6 +1596,7 @@ gsr_default_schedule_handler( GNCSplitReg *gsr, gpointer data )
         ((guid_equal (xaccSchedXactionGetGUID (sx), fromSXId))
          ? sx : NULL);
     }
+    guid_free (fromSXId);
 
     if ( theSX )
     {
@@ -1645,7 +1876,7 @@ gnc_split_reg_jump_cb( GtkWidget *widget, gpointer data )
 }
 
 void
-gnc_split_reg_change_style (GNCSplitReg *gsr, SplitRegisterStyle style)
+gnc_split_reg_change_style (GNCSplitReg *gsr, SplitRegisterStyle style, gboolean refresh)
 {
     SplitRegister *reg = gnc_ledger_display_get_split_register (gsr->ledger);
 
@@ -1653,7 +1884,8 @@ gnc_split_reg_change_style (GNCSplitReg *gsr, SplitRegisterStyle style)
         return;
 
     gnc_split_register_config (reg, reg->type, style, reg->use_double_line);
-    gnc_ledger_display_refresh (gsr->ledger);
+    if (refresh)
+        gnc_ledger_display_refresh (gsr->ledger);
 }
 
 void
@@ -1664,7 +1896,7 @@ gnc_split_reg_style_ledger_cb (GtkWidget *w, gpointer data)
     if (!gtk_check_menu_item_get_active (GTK_CHECK_MENU_ITEM(w)))
         return;
 
-    gnc_split_reg_change_style (gsr, REG_STYLE_LEDGER);
+    gnc_split_reg_change_style (gsr, REG_STYLE_LEDGER, TRUE);
 }
 
 void
@@ -1675,7 +1907,7 @@ gnc_split_reg_style_auto_ledger_cb (GtkWidget *w, gpointer data)
     if (!gtk_check_menu_item_get_active (GTK_CHECK_MENU_ITEM(w)))
         return;
 
-    gnc_split_reg_change_style (gsr, REG_STYLE_AUTO_LEDGER);
+    gnc_split_reg_change_style (gsr, REG_STYLE_AUTO_LEDGER, TRUE);
 }
 
 void
@@ -1686,7 +1918,7 @@ gnc_split_reg_style_journal_cb (GtkWidget *w, gpointer data)
     if (!gtk_check_menu_item_get_active (GTK_CHECK_MENU_ITEM(w)))
         return;
 
-    gnc_split_reg_change_style (gsr, REG_STYLE_JOURNAL);
+    gnc_split_reg_change_style (gsr, REG_STYLE_JOURNAL, TRUE);
 }
 
 void
@@ -1857,16 +2089,18 @@ gnc_split_reg_sort_notes_cb(GtkWidget *w, gpointer data)
 
 
 void
-gnc_split_reg_set_sort_reversed(GNCSplitReg *gsr, gboolean rev)
+gnc_split_reg_set_sort_reversed(GNCSplitReg *gsr, gboolean rev, gboolean refresh)
 {
-  /* Note: sort_reversed is the boolean opposite of sort_increasing
-   *       so when rev == true, we're sorting decreasing
-   *       In other words, qof_query_set_sort_increasing should
-   *       always use the inverse of rev.
-   */
-  Query *query = gnc_ledger_display_get_query( gsr->ledger );
-  qof_query_set_sort_increasing (query, !rev, !rev, !rev);
-  gnc_ledger_display_refresh( gsr->ledger );
+    /* Note: sort_reversed is the boolean opposite of sort_increasing
+     *       so when rev == true, we're sorting decreasing
+     *       In other words, qof_query_set_sort_increasing should
+     *       always use the inverse of rev.
+     */
+    Query *query = gnc_ledger_display_get_query( gsr->ledger );
+    qof_query_set_sort_increasing (query, !rev, !rev, !rev);
+    gsr->sort_rev = rev;
+    if (refresh)
+        gnc_ledger_display_refresh( gsr->ledger );
 }
 
 static void
@@ -1994,14 +2228,17 @@ gnc_split_reg_size_allocate (GtkWidget *widget,
 
 static
 GtkWidget*
-add_summary_label (GtkWidget *summarybar, const char *label_str)
+add_summary_label (GtkWidget *summarybar, gboolean pack_start, const char *label_str, GtkWidget *extra)
 {
     GtkWidget *hbox;
     GtkWidget *label;
 
     hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 2);
     gtk_box_set_homogeneous (GTK_BOX (hbox), FALSE);
-    gtk_box_pack_start( GTK_BOX(summarybar), hbox, FALSE, FALSE, 5 );
+    if (pack_start)
+        gtk_box_pack_start( GTK_BOX(summarybar), hbox, FALSE, FALSE, 5 );
+    else
+        gtk_box_pack_end( GTK_BOX(summarybar), hbox, FALSE, FALSE, 5 );
 
     label = gtk_label_new( label_str );
     gnc_label_set_alignment(label, 1.0, 0.5 );
@@ -2011,44 +2248,65 @@ add_summary_label (GtkWidget *summarybar, const char *label_str)
     gnc_label_set_alignment(label, 1.0, 0.5 );
     gtk_box_pack_start( GTK_BOX(hbox), label, FALSE, FALSE, 0 );
 
+    if (extra != NULL)
+        gtk_box_pack_start( GTK_BOX(hbox), extra, FALSE, FALSE, 0 );
+
     return label;
 }
 
+static void
+gsr_summarybar_set_arrow_draw (GNCSplitReg *gsr)
+{
+    if (gsr->sort_arrow_handler_id > 0)
+       g_signal_handler_disconnect (G_OBJECT(gsr->sort_arrow), gsr->sort_arrow_handler_id);
+
+    gsr->sort_arrow_handler_id = g_signal_connect (G_OBJECT (gsr->sort_arrow), "draw",
+                                                   G_CALLBACK (gnc_draw_arrow_cb), GINT_TO_POINTER(gsr->sort_rev));
+
+    gtk_widget_queue_draw (gsr->sort_arrow);
+}
+
 GtkWidget *
 gsr_create_summary_bar( GNCSplitReg *gsr )
 {
-    GtkWidget *summarybar;
+    GtkWidget *summarybar = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 4);
+    gtk_box_set_homogeneous (GTK_BOX (summarybar), FALSE);
+    gtk_widget_set_name (summarybar, "gnc-id-summarybar");
 
     gsr->cleared_label    = NULL;
     gsr->balance_label    = NULL;
     gsr->reconciled_label = NULL;
     gsr->future_label     = NULL;
     gsr->projectedminimum_label  = NULL;
+    gsr->sort_label       = NULL;
+    gsr->sort_arrow       = NULL;
+    gsr->filter_label     = NULL;
     gsr->shares_label     = NULL;
     gsr->value_label      = NULL;
 
-    if ( gnc_ledger_display_type(gsr->ledger) >= LD_SUBACCOUNT )
+    if (gnc_ledger_display_type(gsr->ledger) == LD_SINGLE)
     {
-        gsr->summarybar = NULL;
-        return NULL;
+        if (!xaccAccountIsPriced(gnc_ledger_display_leader(gsr->ledger)))
+        {
+            gsr->balance_label    = add_summary_label (summarybar, TRUE, _("Present:"), NULL);
+            gsr->future_label     = add_summary_label (summarybar, TRUE, _("Future:"), NULL);
+            gsr->cleared_label    = add_summary_label (summarybar, TRUE, _("Cleared:"), NULL);
+            gsr->reconciled_label = add_summary_label (summarybar, TRUE, _("Reconciled:"), NULL);
+            gsr->projectedminimum_label  = add_summary_label (summarybar, TRUE, _("Projected Minimum:"), NULL);
+        }
+        else
+        {
+            gsr->shares_label     = add_summary_label (summarybar, TRUE, _("Shares:"), NULL);
+            gsr->value_label      = add_summary_label (summarybar, TRUE, _("Current Value:"), NULL);
+        }
     }
 
-    summarybar = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 4);
-    gtk_box_set_homogeneous (GTK_BOX (summarybar), FALSE);
+    gsr->filter_label = add_summary_label (summarybar, FALSE, "", NULL);
+    gsr->sort_arrow = gtk_image_new_from_icon_name ("image-missing", GTK_ICON_SIZE_SMALL_TOOLBAR);
+    gsr->sort_label = add_summary_label (summarybar, FALSE, _("Sort By: "), gsr->sort_arrow);
 
-    if (!xaccAccountIsPriced(gnc_ledger_display_leader(gsr->ledger)))
-    {
-        gsr->balance_label    = add_summary_label (summarybar, _("Present:"));
-        gsr->future_label     = add_summary_label (summarybar, _("Future:"));
-        gsr->cleared_label    = add_summary_label (summarybar, _("Cleared:"));
-        gsr->reconciled_label = add_summary_label (summarybar, _("Reconciled:"));
-        gsr->projectedminimum_label  = add_summary_label (summarybar, _("Projected Minimum:"));
-    }
-    else
-    {
-        gsr->shares_label     = add_summary_label (summarybar, _("Shares:"));
-        gsr->value_label      = add_summary_label (summarybar, _("Current Value:"));
-    }
+    gnc_widget_set_style_context (GTK_WIDGET(gsr->filter_label), "gnc-class-highlight");
+    gnc_widget_set_style_context (GTK_WIDGET(gsr->sort_arrow), "gnc-class-highlight");
 
     gsr->summarybar = summarybar;
 
@@ -2144,7 +2402,6 @@ static
 void
 gnc_split_reg_determine_read_only( GNCSplitReg *gsr )
 {
-    dialog_args *args = g_malloc(sizeof(dialog_args));
     SplitRegister *reg;
 
     if (qof_book_is_readonly(gnc_get_current_book()))
@@ -2156,7 +2413,8 @@ gnc_split_reg_determine_read_only( GNCSplitReg *gsr )
 
     if ( !gsr->read_only )
     {
-
+        dialog_args *args;
+        char *string = NULL;
         switch (gnc_split_reg_get_placeholder(gsr))
         {
         case PLACEHOLDER_NONE:
@@ -2164,14 +2422,14 @@ gnc_split_reg_determine_read_only( GNCSplitReg *gsr )
             return;
 
         case PLACEHOLDER_THIS:
-            args->string = _("This account may not be edited. If you want "
+            string = _("This account may not be edited. If you want "
                              "to edit transactions in this register, please "
                              "open the account options and turn off the "
                              "placeholder checkbox.");
             break;
 
         default:
-            args->string = _("One of the sub-accounts selected may not be "
+            string = _("One of the sub-accounts selected may not be "
                              "edited. If you want to edit transactions in "
                              "this register, please open the sub-account "
                              "options and turn off the placeholder checkbox. "
@@ -2181,6 +2439,8 @@ gnc_split_reg_determine_read_only( GNCSplitReg *gsr )
         }
         gsr->read_only = TRUE;
         /* Put up a warning dialog */
+        args = g_malloc(sizeof(dialog_args));
+        args->string = string;
         args->gsr = gsr;
         g_timeout_add (250, gtk_callback_bug_workaround, args); /* 0.25 seconds */
     }
