@@ -64,6 +64,7 @@ extern "C"
 #include "gnc-tokenizer-fw.hpp"
 #include "gnc-tokenizer-csv.hpp"
 
+#include <gnc-locale-utils.hpp>
 #include <boost/locale.hpp>
 
 namespace bl = boost::locale;
@@ -159,7 +160,8 @@ public:
     void assist_finish ();
     void assist_compmgr_close ();
 
-    void file_confirm_cb ();
+    void file_activated_cb ();
+    void file_selection_changed_cb ();
 
     void preview_settings_delete ();
     void preview_settings_save ();
@@ -202,6 +204,8 @@ private:
     GtkWidget* preview_cbox_factory (GtkTreeModel* model, uint32_t colnum);
     /* helper function to set rendering parameters for preview data columns */
     void preview_style_column (uint32_t col_num, GtkTreeModel* model);
+    /* helper function to check for a valid filename as opposed to a directory */
+    bool check_for_valid_filename ();
 
     GtkAssistant    *csv_imp_asst;
 
@@ -269,7 +273,8 @@ extern "C"
 void csv_tximp_assist_prepare_cb (GtkAssistant  *assistant, GtkWidget *page, CsvImpTransAssist* info);
 void csv_tximp_assist_close_cb (GtkAssistant *gtkassistant, CsvImpTransAssist* info);
 void csv_tximp_assist_finish_cb (GtkAssistant *gtkassistant, CsvImpTransAssist* info);
-void csv_tximp_file_confirm_cb (GtkWidget *button, CsvImpTransAssist *info);
+void csv_tximp_file_activated_cb (GtkFileChooser *chooser,  CsvImpTransAssist *info);
+void csv_tximp_file_selection_changed_cb (GtkFileChooser *chooser,  CsvImpTransAssist *info);
 void csv_tximp_preview_del_settings_cb (GtkWidget *button, CsvImpTransAssist *info);
 void csv_tximp_preview_save_settings_cb (GtkWidget *button, CsvImpTransAssist *info);
 void csv_tximp_preview_settings_sel_changed_cb (GtkComboBox *combo, CsvImpTransAssist *info);
@@ -309,10 +314,14 @@ csv_tximp_assist_finish_cb (GtkAssistant *assistant, CsvImpTransAssist* info)
     info->assist_finish ();
 }
 
-
-void csv_tximp_file_confirm_cb (GtkWidget *button, CsvImpTransAssist *info)
+void csv_tximp_file_activated_cb (GtkFileChooser *chooser, CsvImpTransAssist *info)
 {
-    info->file_confirm_cb();
+    info->file_activated_cb();
+}
+
+void csv_tximp_file_selection_changed_cb (GtkFileChooser *chooser, CsvImpTransAssist *info)
+{
+    info->file_selection_changed_cb();
 }
 
 void csv_tximp_preview_del_settings_cb (GtkWidget *button, CsvImpTransAssist *info)
@@ -479,18 +488,10 @@ CsvImpTransAssist::CsvImpTransAssist ()
     /* File chooser Page */
     file_page = GTK_WIDGET(gtk_builder_get_object (builder, "file_page"));
     file_chooser = gtk_file_chooser_widget_new (GTK_FILE_CHOOSER_ACTION_OPEN);
+    g_signal_connect (G_OBJECT(file_chooser), "selection-changed",
+                      G_CALLBACK(csv_tximp_file_selection_changed_cb), this);
     g_signal_connect (G_OBJECT(file_chooser), "file-activated",
-                      G_CALLBACK(csv_tximp_file_confirm_cb), this);
-    auto button = gtk_button_new_with_mnemonic (_("_OK"));
-    gtk_widget_set_size_request (button, 100, -1);
-    gtk_widget_show (button);
-    auto h_box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
-    gtk_box_set_homogeneous (GTK_BOX (h_box), TRUE);
-    gtk_widget_set_hexpand (GTK_WIDGET(h_box), TRUE);
-    gtk_box_pack_start (GTK_BOX(h_box), button, FALSE, FALSE, 0);
-    gtk_file_chooser_set_extra_widget (GTK_FILE_CHOOSER(file_chooser), h_box);
-    g_signal_connect (G_OBJECT(button), "clicked",
-                      G_CALLBACK(csv_tximp_file_confirm_cb), this);
+                      G_CALLBACK(csv_tximp_file_activated_cb), this);
 
     auto box = GTK_WIDGET(gtk_builder_get_object (builder, "file_page"));
     gtk_box_pack_start (GTK_BOX(box), file_chooser, TRUE, TRUE, 6);
@@ -648,7 +649,8 @@ CsvImpTransAssist::CsvImpTransAssist ()
     summary_page  = GTK_WIDGET(gtk_builder_get_object (builder, "summary_page"));
     summary_label = GTK_WIDGET(gtk_builder_get_object (builder, "summary_label"));
 
-    gnc_restore_window_size (GNC_PREFS_GROUP, GTK_WINDOW(csv_imp_asst));
+    gnc_restore_window_size (GNC_PREFS_GROUP,
+                             GTK_WINDOW(csv_imp_asst), gnc_ui_get_main_window(nullptr));
 
     gtk_builder_connect_signals (builder, this);
     g_object_unref (G_OBJECT(builder));
@@ -680,18 +682,14 @@ CsvImpTransAssist::~CsvImpTransAssist ()
  * Code related to the file chooser page
  **************************************************/
 
-/* csv_tximp_file_confirm_cb
- *
- * call back for ok button in file chooser widget
+/* check_for_valid_filename for a valid file to activate the forward button
  */
-void
-CsvImpTransAssist::file_confirm_cb ()
+bool
+CsvImpTransAssist::check_for_valid_filename ()
 {
-    gtk_assistant_set_page_complete (csv_imp_asst, account_match_page, false);
-
     auto file_name = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER(file_chooser));
-    if (!file_name)
-        return;
+    if (!file_name || g_file_test (file_name, G_FILE_TEST_IS_DIR))
+        return false;
 
     auto filepath = gnc_uri_get_path (file_name);
     auto starting_dir = g_path_get_dirname (filepath);
@@ -706,37 +704,36 @@ CsvImpTransAssist::file_confirm_cb ()
     g_free (file_name);
     g_free (starting_dir);
 
-    /* Load the file into parse_data. */
-    tx_imp = std::unique_ptr<GncTxImport>(new GncTxImport);
-    /* Assume data is CSV. User can later override to Fixed Width if needed */
-    try
-    {
-        tx_imp->file_format (GncImpFileFormat::CSV);
-        tx_imp->load_file (m_file_name);
-        tx_imp->tokenize (true);
-    }
-    catch (std::ifstream::failure& e)
-    {
-        /* File loading failed ... */
-        gnc_error_dialog (GTK_WINDOW (csv_imp_asst), "%s", e.what());
-        return;
-    }
-    catch (std::range_error &e)
-    {
-        /* Parsing failed ... */
-        gnc_error_dialog (GTK_WINDOW (csv_imp_asst), "%s", _(e.what()));
-        return;
-    }
+    return true;
+}
 
-    preview_refresh ();
+/* csv_tximp_file_activated_cb
+ *
+ * call back for file chooser widget
+ */
+void
+CsvImpTransAssist::file_activated_cb ()
+{
+    gtk_assistant_set_page_complete (csv_imp_asst, file_page, false);
 
-    /* Get settings store and populate */
-    preview_populate_settings_combo();
-    gtk_combo_box_set_active (settings_combo, 0);
+    /* Test for a valid filename and not a directory */
+    if (check_for_valid_filename ())
+    {
+        gtk_assistant_set_page_complete (csv_imp_asst, file_page, true);
+        gtk_assistant_next_page (csv_imp_asst);
+    }
+}
 
-    gtk_assistant_set_page_complete (csv_imp_asst, account_match_page, true);
-    auto num = gtk_assistant_get_current_page (csv_imp_asst);
-    gtk_assistant_set_current_page (csv_imp_asst, num + 1);
+/* csv_tximp_file_selection_changed_cb
+ *
+ * call back for file chooser widget
+ */
+void
+CsvImpTransAssist::file_selection_changed_cb ()
+{
+    /* Enable the forward button based on a valid filename */
+    gtk_assistant_set_page_complete (csv_imp_asst, file_page,
+        check_for_valid_filename ());
 }
 
 
@@ -1658,13 +1655,20 @@ CsvImpTransAssist::preview_refresh ()
     go_charmap_sel_set_encoding (encselector, tx_imp->encoding().c_str());
 
     // Handle separator checkboxes and custom field, only relevant if the file format is csv
+    // Note we defer the change signal until all buttons have been updated
+    // An early update may result in an incomplete tokenize run and that would
+    // cause our list of saved column types to be truncated
     if (tx_imp->file_format() == GncImpFileFormat::CSV)
     {
         auto separators = tx_imp->separators();
         const auto stock_sep_chars = std::string (" \t,:;-");
         for (int i = 0; i < SEP_NUM_OF_TYPES; i++)
+        {
+            g_signal_handlers_block_by_func (sep_button[i], (gpointer) csv_tximp_preview_sep_button_cb, this);
             gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(sep_button[i]),
                 separators.find (stock_sep_chars[i]) != std::string::npos);
+            g_signal_handlers_unblock_by_func (sep_button[i], (gpointer) csv_tximp_preview_sep_button_cb, this);
+        }
 
         // If there are any other separators in the separators string,
         // add them as custom separators
@@ -1674,9 +1678,14 @@ CsvImpTransAssist::preview_refresh ()
             separators.erase(pos);
             pos = separators.find_first_of (stock_sep_chars);
         }
+        g_signal_handlers_block_by_func (custom_cbutton, (gpointer) csv_tximp_preview_sep_button_cb, this);
+        g_signal_handlers_block_by_func (custom_entry, (gpointer) csv_tximp_preview_sep_button_cb, this);
         gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(custom_cbutton),
-                !separators.empty());
+                                      !separators.empty());
         gtk_entry_set_text (GTK_ENTRY(custom_entry), separators.c_str());
+        g_signal_handlers_unblock_by_func (custom_cbutton, (gpointer) csv_tximp_preview_sep_button_cb, this);
+        g_signal_handlers_unblock_by_func (custom_entry, (gpointer) csv_tximp_preview_sep_button_cb, this);
+        csv_tximp_preview_sep_button_cb (GTK_WIDGET (custom_cbutton), this);
     }
 
     // Repopulate the parsed data table
@@ -1695,9 +1704,11 @@ void CsvImpTransAssist::preview_validate_settings ()
 
     /* Show or hide the account match page based on whether there are
      * accounts in the user data according to the importer configuration
+     * only if there are no errors
      */
-    gtk_widget_set_visible (GTK_WIDGET(account_match_page),
-            !tx_imp->accounts().empty());
+    if (error_msg.empty())
+        gtk_widget_set_visible (GTK_WIDGET(account_match_page),
+                !tx_imp->accounts().empty());
 }
 
 
@@ -1778,7 +1789,6 @@ csv_tximp_acct_match_text_parse (std::string acct_name)
             alt_sep = "-";
         else
             alt_sep = ":";
-        sep_pos = acct_name.find(sep);
         for (sep_pos = acct_name.find(sep); sep_pos != std::string::npos;
                 sep_pos = acct_name.find(sep))
             acct_name.replace (sep_pos, strlen(sep), alt_sep);
@@ -1883,13 +1893,52 @@ CsvImpTransAssist::assist_file_page_prepare ()
 void
 CsvImpTransAssist::assist_preview_page_prepare ()
 {
-    tx_imp->req_mapped_accts (false);
+    auto go_back = false;
 
-    /* Disable the Forward Assistant Button */
-    gtk_assistant_set_page_complete (csv_imp_asst, preview_page, false);
+    /* Load the file into parse_data, reset if already loaded. */
+    if (tx_imp)
+        tx_imp.reset();
 
-    /* Load the data into the treeview. */
-    preview_refresh_table ();
+    tx_imp = std::unique_ptr<GncTxImport>(new GncTxImport);
+
+    /* Assume data is CSV. User can later override to Fixed Width if needed */
+    try
+    {
+        tx_imp->file_format (GncImpFileFormat::CSV);
+        tx_imp->load_file (m_file_name);
+        tx_imp->tokenize (true);
+    }
+    catch (std::ifstream::failure& e)
+    {
+        /* File loading failed ... */
+        gnc_error_dialog (GTK_WINDOW (csv_imp_asst), "%s", e.what());
+        go_back = true;
+    }
+    catch (std::range_error &e)
+    {
+        /* Parsing failed ... */
+        gnc_error_dialog (GTK_WINDOW (csv_imp_asst), "%s", _(e.what()));
+        go_back = true;
+    }
+
+    if (go_back)
+        gtk_assistant_previous_page (csv_imp_asst);
+    else
+    {
+        preview_refresh ();
+
+        /* Get settings store and populate */
+        preview_populate_settings_combo();
+        gtk_combo_box_set_active (settings_combo, 0);
+
+        tx_imp->req_mapped_accts (false);
+
+        /* Disable the Forward Assistant Button */
+        gtk_assistant_set_page_complete (csv_imp_asst, preview_page, false);
+
+        /* Load the data into the treeview. */
+        g_idle_add ((GSourceFunc)csv_imp_preview_queue_rebuild_table, this);
+    }
 }
 
 void
@@ -1914,7 +1963,7 @@ CsvImpTransAssist::assist_account_match_page_prepare ()
     gtk_widget_set_sensitive (account_match_btn, true);
 
     /* Enable the Forward Assistant Button */
-       gtk_assistant_set_page_complete (csv_imp_asst, account_match_page,
+    gtk_assistant_set_page_complete (csv_imp_asst, account_match_page,
                csv_tximp_acct_match_check_all (store));
 }
 
@@ -2023,15 +2072,28 @@ CsvImpTransAssist::assist_summary_page_prepare ()
     gtk_assistant_remove_action_widget (csv_imp_asst, help_button);
     gtk_assistant_remove_action_widget (csv_imp_asst, cancel_button);
 
-    bl::generator gen;
-    gen.add_messages_path(gnc_path_get_datadir());
-    gen.add_messages_domain(PACKAGE);
-
     // FIXME Rather than passing a locale generator below we probably should set std::locale::global appropriately somewhere.
+    bl::generator gen;
+    gen.add_messages_path(gnc_path_get_localedir());
+    gen.add_messages_domain(GETTEXT_PACKAGE);
+
     auto text = std::string("<span size=\"medium\"><b>");
+    try
+    {
     /* Translators: {1} will be replaced with a filename */
-    text += (bl::format (bl::translate ("The transactions were imported from file '{1}'.")) % m_file_name).str(gen(""));
-    text += "</b></span>";
+      text += (bl::format (bl::translate ("The transactions were imported from file '{1}'.")) % m_file_name).str(gnc_get_locale());
+        text += "</b></span>";
+    }
+    catch (const bl::conv::conversion_error& err)
+    {
+        PERR("Transcoding error: %s", err.what());
+        text += "The transactions were imported from the file.</b></span>";
+    }
+    catch (const bl::conv::invalid_charset_error& err)
+    {
+        PERR("Invalid charset error: %s", err.what());
+        text += "The transactions were imported from the file.</b></span>";
+    }
     gtk_label_set_markup (GTK_LABEL(summary_label), text.c_str());
 }
 
