@@ -7,12 +7,8 @@
 ;; otherwise the N_ syntax-rule won't be found at compile time
 ;; causing the test to fail
 ;; That's what the wrapper below is meant for:
-(cond-expand
-   (guile-2
-    (define-syntax-rule (begin-for-syntax form ...)
-      (eval-when (load compile eval) (begin form ...))))
-   (else
-    (define begin-for-syntax begin)))
+(define-syntax-rule (begin-for-syntax form ...)
+      (eval-when (load compile eval) (begin form ...)))
 
 (begin-for-syntax (define loaded-module (gnc:module-load "gnucash/app-utils" 0)))
 (if loaded-module
@@ -21,10 +17,10 @@
       (display "Failed - module gnucash/app-utils not loaded successfully\n")
       (set! exit-code -1)))
 
-(if (procedure? gnc:error->string)
-    (display "Procedure gnc:error->string found\n")
+(if (procedure? gnc:apply-with-error-handling)
+    (display "Procedure gnc:apply-with-error-handling found\n")
     (begin
-      (display "Failed - procedure gnc:error->string not found\n")
+      (display "Failed - procedure gnc:apply-with-error-handling not found\n")
       (set! exit-code -1)))
 
 (if (procedure? gnc-default-currency)
